#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pyrmap.h"

void make_solid_map(struct dungeon_map *map, int x, int y)
{
    int sz = x * y / 8;
    map->x = x;
    map->y = y;
    map->data = malloc(sz);
    memset(map->data, 0xff, sz);
}

void clear_map(struct dungeon_map *map)
{
    free(map->data);
}

int get_room(const struct dungeon_map *map, int x, int y)
{
    int byte_n, bit_n;
    if(x < 0 || x >= map->x || y < 0 || y > map->y)
        return 0;

    byte_n = y * (map->x/8) + x/8;
    bit_n = 7 - x % 8;
    return (map->data[byte_n] & (1<<bit_n)) ? 1 : 0;
}

void set_room(const struct dungeon_map *map, int x, int y, int val)
{
    int byte_n, bit_n, mask;
    if(x < 0 || x >= map->x || y < 0 || y > map->y)
        return;

    byte_n = y * (map->x/8) + x/8;
    bit_n = 7 - x % 8;

    mask = 1 << bit_n;

    if(val)
        map->data[byte_n] |= mask;
    else
        map->data[byte_n] &= ~mask;
}

void set_region(const struct dungeon_map *map,
                int x, int y, int m, int n, int val)
{
    int i, j;
    for(i = 0; i < m; i++)
        for(j = 0; j < n; j++)
            set_room(map, x+i, y+j, val);
}

void print_map(const struct dungeon_map *map)
{
    int i, j;
    for(j = 0; j < map->y; j++) {
        putchar('@');
        putchar(' ');
        for(i = 0; i < map->x; i++)
            putchar(get_room(map, i, j) ? '#' : '.');
        putchar('\n');
    }
}

int save_map(const struct dungeon_map *map, const char *fname)
{
    int i, sz, lns;
    FILE *f = fopen(fname, "w");
    if(!f)
        return 0;

    sz = map->x * map->y / 8;
    lns = map->x/8;
    for(i = 0; i < sz; i++) {
        fprintf(f, "%02x", map->data[i]);
        if(i % 70 == 69 || i % lns == lns-1)
            fputc('\n', f);
        else
        if(i % 8 == 7)
            fputc(' ', f);
    }
    fputc('\n', f);
    fclose(f);
    return 1;
}

